

C     ==============
      program durat2
C     ==============

C--
C--   Three ways to calculate the error on the d/u ratio
C--

      implicit double precision (a-h,o-z)

      parameter (nwpawc = 1000000)
      common /pawc/ hmemor(nwpawc)
     
      common /xqgri/ xxtab(100),qqtab(100),nxx,nqq
      common /fitpa/ aa(50),d1(50),d2(50),d3(50),ifit(50),npar

      real t1,t2,t3

      external rana,rnum

      dimension dn(-5:5),up(-5:5)

C--            bb  cb  sb  ub  db   g   d   u   s   c   b
      data dn / 0., 0., 0., 0., 1., 0., 1., 0., 0., 0., 0. /
      data up / 0., 0., 0., 1., 0., 0., 0., 1., 0., 0., 0. /

C--   Hbook initialisation
      call hlimit(nwpawc)

C--   x-Q2 table for plotting
      nxx        = 10
      xxtab(1)   = 1.D-3
      xxtab(nxx) = 0.9
      nqq        = 1
      qqtab(1)   = 10.D0
      call maketab

C--   Stop/print
      call ep_stop(1,0)

C--   Open input ntuple file
      call timex(t1)
      call ep_open(20,'../fits/mbfit1m.pdfnt2',ierr)

C--   Get QCD fit parameters (for s/r rnum)
      call ep_pars(aa,d1,d2,d3,ifit,50,npar)

C--   Read dn and up distributions
      call ep_read(1, 0, 0, dn, ierr )
      call ep_read(2, 0, 0, up, ierr )

C--   Select error
      mode = 2

C--   Now for the result
      call timex(t2)
      write(6,1000)
      do ix = 1,nxx
        do iq = 1,nqq

          xx  = xxtab(ix)
          qq  = qqtab(iq)

C--       Central value
          fd   = ep_valu(1,xx,qq,ierr)
          fu   = ep_valu(2,xx,qq,ierr)
          ff   = fd/fu

C--       Error: use covariance matrix
          vdd  = ep_cvar(1,xx,qq,1,xx,qq,mode,ierr)
          vdu  = ep_cvar(1,xx,qq,2,xx,qq,mode,ierr)
          vuu  = ep_cvar(2,xx,qq,2,xx,qq,mode,ierr)
          e1   = raterr(fd,fu,vdd,vdu,vuu)

C--       Error: use rana (analytical calculation of df/dpar)
          e2   = ep_uerr(rana,xx,qq,mode,ierr)

C--       Error: use rnum (numerical  calculation of df/dpar)
          e3   = ep_uerr(rnum,xx,qq,mode,ierr)

          write(6,'(I5,2E11.3,3E12.4)') ix,xx,ff,e1,e2,e3

        enddo
      enddo
      call timex(t3)

      write(6,'(/'' Time spent init ='',F6.2)') t2-t1
      write(6,'( ''            loop ='',F6.2)') t3-t2
      write(6,'( ''           total ='',F6.2)') t3-t1

 1000 format(/'    #       x        d/u        err1',
     +        '        err2        err3')

      end

C     =================================================
      double precision function raterr(a,b,v11,v12,v22)
C     =================================================

C--   Error propagation on ratio

      implicit double precision (a-h,o-z)

      raterr = 0.D0

      if(b.eq.0.D0) return

      f  = a/b
      fa = f/a
      fb = f/b
      d2 = fa*fa*v11 - 2.*fa*fb*v12 + fb*fb*v22

      raterr = sqrt(d2)

      return
      end

C     =============================================
      double precision function rana(ider,x,q,ierr)
C     =============================================

C--   Calculate u/d or d/dpar [u/d] analytically

      implicit double precision (a-h,o-z)

      rana   = 0.D0

C--   Central value
      dn     = ep_allf(1, 0  ,x,q,ierr)
      up     = ep_allf(2, 0  ,x,q,ierr)

      if(dn.eq.0.D0.or.up.eq.0.D0) return

      rana   = dn/up

      if(ider.eq.0)                return

C--   df/dpar (analytically)
      dnder  = ep_allf(1,ider,x,q,ierr)
      upder  = ep_allf(2,ider,x,q,ierr)
      rana   = rana * ( dnder/dn - upder/up )
    
      return
      end

C     =============================================
      double precision function rnum(ider,x,q,ierr)
C     =============================================

C--   Calculate u/d or d/dpar [u/d] numerically

      implicit double precision (a-h,o-z)

      common /fitpa/ aa(50),d1(50),d2(50),d3(50),ifit(50),npar

      rnum   = 0.D0

C--   Central value
      dn     = ep_allf(1, 0  ,x,q,ierr)
      up     = ep_allf(2, 0  ,x,q,ierr)

      if(dn.eq.0.D0.or.up.eq.0.D0) return

      rnum   = dn/up

      if(ider.eq.0)                return

C--   df/dpar (numerically)
      dnder  = ep_allf(1,ider,x,q,ierr)
      dnplu  = dn + d1(ider)*dnder
      dnmin  = dn - d1(ider)*dnder
      upder  = ep_allf(2,ider,x,q,ierr)
      upplu  = up + d1(ider)*upder
      upmin  = up - d1(ider)*upder
      rrplu  = dnplu/upplu
      rrmin  = dnmin/upmin
      rnum   = (rrplu-rrmin)/(2.*d1(ider))
    
      return
      end

C     ==================
      subroutine maketab
C     ==================

C--   Generate a logarithmic table of x and Q2 values

      implicit double precision (a-h,o-z)

      common /xqgri/ xxtab(100),qqtab(100),nxx,nqq

      if(nxx.gt.1) then
        xmi  = xxtab(1)
        xma  = xxtab(nxx)
        xmil = log(xmi)
        xmal = log(xma)
        bw   = (xmal-xmil)/(nxx-1)
        do i = 1,nxx
          xl       = xmil + (i-1)*bw
          xxtab(i) = exp(xl)
        enddo
      endif

      if(nqq.gt.1) then
        qmi  = qqtab(1)
        qma  = qqtab(nqq)
        qmil = log(qmi)
        qmal = log(qma)
        bw   = (qmal-qmil)/(nqq-1)
        do i = 1,nqq
          ql       = qmil + (i-1)*bw
          qqtab(i) = exp(ql)
        enddo
      endif

      return
      end
