
C     ============
      program fcc2
C     ============

C--
C--   Calculate CC cross structure functions at HERA
C--   See ZN 99-027
C--
C--   Note: only valid for Q2 > Q2_c (= 4 GeV2)
C--         For lower Q2 CKM matrix elements have to be included
C--

      implicit double precision (a-h,o-z)

      parameter (nwpawc = 1000000)
      common /pawc/ hmemor(nwpawc)

      common /flagcm/ istf,isign,pol
      common /xqgrid/ xxtab(100),qqtab(100),nxx,nqq
     
      external getfcc

      real t1,t2,t3

      dimension dn(-5:5),db(-5:5),up(-5:5),ub(-5:5)
C--   No contribution from b and bbar (Cabbibo suppressed)
C--            bb  cb  sb  ub  db   g   d   u   s   c   b
      data dn / 0., 0., 0., 0., 0., 0., 1., 0., 1., 0., 0. /
      data db / 0., 0., 1., 0., 1., 0., 0., 0., 0., 0., 0. /
      data up / 0., 0., 0., 0., 0., 0., 0., 1., 0., 1., 0. /
      data ub / 0., 1., 0., 1., 0., 0., 0., 0., 0., 0., 0. /

C--   Choices
      istf     =  1       ! 1,2,3 for F2,FL,xF3
      isign    = -1       ! -1,+1 for electrons, positrons
      pol      =  0.      ! -1 .le. pol .le. +1 (polarization)

C--   Hbook initialisation
      call hlimit(nwpawc)

C--   x-Q2 table for plotting
      nxx        = 10
      xxtab(1)   = 1.D-3
      xxtab(nxx) = 0.9
      nqq        = 1
      qqtab(1)   = 10.D0
      call maketab

C--   Stop/print
      call ep_stop(1,0)

C--   Open stf file
      call timex(t1)
      call ep_open(20,'../fits/mbfit1m.stfnt2',ierr)

C--   Read dntype, uptype
      call ep_read( 1, istf, 0, dn, ierr )
      call ep_read( 2, istf, 0, db, ierr )
      call ep_read( 3, istf, 0, up, ierr )
      call ep_read( 4, istf, 0, ub, ierr )

C--   Select error
      mode = 2

C--   Now for the result

      write(6,1000)

      call timex(t2)
      do ix = 1,nxx
        do iq = 1,nqq

          xx  = xxtab(ix)
          qq  = qqtab(iq)

          ff   = ep_ufun(getfcc,xx,qq,     ierr)
          ef   = ep_uerr(getfcc,xx,qq,mode,ierr)

          write(6,'(I5,3E11.3)') ix,xx,ff,ef

        enddo
      enddo
      call timex(t3)

      write(6,'(/'' Time spent init ='',F6.2)') t2-t1
      write(6,'( ''            loop ='',F6.2)') t3-t2
      write(6,'( ''           total ='',F6.2)') t3-t1

 1000 format(/'    #       x        fcc        err')

      end

C     ================================================
      double precision function getfcc(ipder,x,q,ierr)
C     ================================================

      implicit double precision (a-h,o-z)

C--   ep CC structure functions, see ZN 99-027

C--   ipder = fit parameter index
C--   istf  = 1,2,3 for F2, FL, xF3
C--   isign = electron sign
C--   pol   = electron polarisation
C--   x     = value of x
C--   q     = value of Q2
C--   ierr  = output error flag

      common /flagcm/ istf,isign,pol

C--   NB 1=dn 2=db 3=up 4=ub

      if(isign.eq.1) then
        if(istf.eq.1.or.istf.eq.2) then
C--       F2,L^+
          getfcc = (1.+pol)*(ep_allf(1,ipder,x,q,ierr) +
     +                       ep_allf(4,ipder,x,q,ierr))
        else
C--       xF3^+
          getfcc = (1.+pol)*(ep_allf(1,ipder,x,q,ierr) -
     +                       ep_allf(4,ipder,x,q,ierr))
        endif
      else
        if(istf.eq.1.or.istf.eq.2) then
C--       F2,L^-
          getfcc = (1.-pol)*(ep_allf(3,ipder,x,q,ierr) +
     +                       ep_allf(2,ipder,x,q,ierr))
        else
C--       xF3^-
          getfcc = (1.-pol)*(ep_allf(3,ipder,x,q,ierr) -
     +                       ep_allf(2,ipder,x,q,ierr))
        endif
      endif

      return
      end

C     ==================
      subroutine maketab
C     ==================

C--   Generate a logarithmic table of x and Q2 values

      implicit double precision (a-h,o-z)

      common /xqgrid/ xxtab(100),qqtab(100),nxx,nqq

      if(nxx.gt.1) then
        xmi  = xxtab(1)
        xma  = xxtab(nxx)
        xmil = log(xmi)
        xmal = log(xma)
        bw   = (xmal-xmil)/(nxx-1)
        do i = 1,nxx
          xl       = xmil + (i-1)*bw
          xxtab(i) = exp(xl)
        enddo
      endif

      if(nqq.gt.1) then
        qmi  = qqtab(1)
        qma  = qqtab(nqq)
        qmil = log(qmi)
        qmal = log(qma)
        bw   = (qmal-qmil)/(nqq-1)
        do i = 1,nqq
          ql       = qmil + (i-1)*bw
          qqtab(i) = exp(ql)
        enddo
      endif

      return
      end
