

C     =============
      program fint2
C     =============

C--
C--   Calculate sumrules, integrals and differentials.
C--

      implicit double precision (a-h,o-z)

      parameter (nwpawc = 1000000)
      common /pawc/ hmemor(nwpawc)
     
      common /pass/ id

      real t1,t2,t3

      external fxdx,fdlx
      external fqdq,fdlq
      external dfdx,dlnx,dfdq,dlnq

      dimension gl(-5:5),si(-5:5),pa(-5:5),dv(-5:5),uv(-5:5)

C--            bb  cb  sb  ub  db   g   d   u   s   c   b
      data gl / 0., 0., 0., 0., 0., 1., 0., 0., 0., 0., 0. /
      data si / 1., 1., 1., 1., 1., 0., 1., 1., 1., 1., 1. /
      data pa / 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1. /
      data dv / 0., 0., 0., 0.,-1., 0., 1., 0., 0., 0., 0. /
      data uv / 0., 0., 0.,-1., 0., 0., 0., 1., 0., 0., 0. /

C--   Hbook initialisation
      call hlimit(nwpawc)

C--   Stop/print
      call ep_stop(1,0)

C--   Open input ntuple file
      call timex(t1)
      call ep_open(20,'../fits/mbfit1m.pdfnt2',ierr)

C--   Read gluon, singlet, total distributions
      call ep_read(1, 0, 0, gl, ierr )
      call ep_read(2, 0, 0, si, ierr )
      call ep_read(3, 0, 0, pa, ierr )
      call ep_read(4, 0, 0, dv, ierr )
      call ep_read(5, 0, 0, uv, ierr )

C--   Select error
      mode = 3

      call timex(t2)

C--   Now for the integrals in x
C--   Lower limit in x
      xx = 1.D-3
      q2 = 4.D0
      write(6,'(/'' Sumrule from xmin ='',E12.3,
     +           '' at Q2 = '',E12.3/)') xx,q2
      do id = 1,3
        val = ep_ufun(fxdx,xx,q2,ierr)
        err = ep_uerr(fxdx,xx,q2,mode,ierr)
        write(6,'(I5,2f10.3)') id,val,err
      enddo
      write(6,'(/)')
      do id = 4,5
        val = ep_ufun(fdlx,xx,q2,ierr)
        err = ep_uerr(fdlx,xx,q2,mode,ierr)
        write(6,'(I5,2f10.3)') id,val,err
      enddo

C--   Now for the integrals in Q2
C--   Upper limit in Q2
      xx = 1.D-1
      q2 = 1000.D0
      write(6,'(/'' Integral upto qmax ='',E12.3,
     +           '' at x = '',E12.3/)') q2,xx
      do id = 1,3
        val = ep_ufun(fqdq,xx,q2,ierr)
        err = ep_uerr(fqdq,xx,q2,mode,ierr)
        write(6,'(I5,2f10.3)') id,val,err
      enddo
      write(6,'(/)')
      do id = 4,5
        val = ep_ufun(fdlq,xx,q2,ierr)
        err = ep_uerr(fdlq,xx,q2,mode,ierr)
        write(6,'(I5,2f10.3)') id,val,err
      enddo

C--   Various differentials
      xx = 1.D-1
      q2 = 10.D0
      write(6,'(/'' Differentials at x, Q2 ='',2E12.5)') xx,q2
      write(6,'(/''     df/dx     error   df/dlnx     error'',
     +           ''    df/dQ2     error  df/dlnQ2     error'')')
      do id = 1,5
        val1 = ep_ufun(dfdx,xx,q2,ierr)
        err1 = ep_uerr(dfdx,xx,q2,mode,ierr)
        val2 = ep_ufun(dlnx,xx,q2,ierr)
        err2 = ep_uerr(dlnx,xx,q2,mode,ierr)
        val3 = ep_ufun(dfdq,xx,q2,ierr)
        err3 = ep_uerr(dfdq,xx,q2,mode,ierr)
        val4 = ep_ufun(dlnq,xx,q2,ierr)
        err4 = ep_uerr(dlnq,xx,q2,mode,ierr)
        write(6,'(8E10.2)') val1,err1,val2,err2,val3,err3,
     +                      val4,err4
      enddo

      call timex(t3)

      write(6,'(/'' Time spent init ='',F6.2)') t2-t1
      write(6,'( ''            loop ='',F6.2)') t3-t2
      write(6,'( ''           total ='',F6.2)') t3-t1

      end

C     ===============================================
      double precision function fxdx(ipder,x,q2,ierr)
C     ===============================================

C--   Int_x^1 xf(x,Q2) dx

      implicit double precision (a-h,o-z)

      common /pass/ id

      a1 = ep_ifdx(id,ipder,1.D0,q2,ierr)
      a2 = ep_ifdx(id,ipder,x   ,q2,ierr)

      fxdx = a1-a2

      return
      end

C     ===============================================
      double precision function fdlx(ipder,x,q2,ierr)
C     ===============================================

C--   Int_x^1 xf(x,Q2) dx

      implicit double precision (a-h,o-z)

      common /pass/ id

      a1 = ep_ilnx(id,ipder,1.D0,q2,ierr)
      a2 = ep_ilnx(id,ipder,x   ,q2,ierr)

      fdlx = a1-a2

      return
      end

C     ===============================================
      double precision function fqdq(ipder,x,q2,ierr)
C     ===============================================

C--   Int_qmin^q xf(x,Q2) dQ2

      implicit double precision (a-h,o-z)

      common /pass/ id

      fqdq = ep_ifdq(id,ipder,x   ,q2,ierr)

      return
      end

C     ===============================================
      double precision function fdlq(ipder,x,q2,ierr)
C     ===============================================

C--   Int_qmin^q xf(x,Q2) dlnQ2

      implicit double precision (a-h,o-z)

      common /pass/ id

      fdlq = ep_ilnq(id,ipder,x   ,q2,ierr)

      return
      end

C     ===============================================
      double precision function dfdx(ipder,x,q2,ierr)
C     ===============================================

C--   df/dx

      implicit double precision (a-h,o-z)

      common /pass/ id

      dfdx = ep_dfdx(id,ipder,x   ,q2,ierr)

      return
      end

C     ===============================================
      double precision function dlnx(ipder,x,q2,ierr)
C     ===============================================

C--   df/dlnx

      implicit double precision (a-h,o-z)

      common /pass/ id

      dlnx = ep_dlnx(id,ipder,x   ,q2,ierr)

      return
      end

C     ===============================================
      double precision function dfdq(ipder,x,q2,ierr)
C     ===============================================

C--   df/dQ2

      implicit double precision (a-h,o-z)

      common /pass/ id

      dfdq = ep_dfdq(id,ipder,x   ,q2,ierr)

      return
      end

C     ===============================================
      double precision function dlnq(ipder,x,q2,ierr)
C     ===============================================

C--   df/dlnQ2

      implicit double precision (a-h,o-z)

      common /pass/ id

      dlnq = ep_dlnq(id,ipder,x   ,q2,ierr)

      return
      end
