
C     ============
      program fnc2
C     ============

C--
C--   Calculate NC structure functions at HERA
C--   See ZN 99-027
C--

      implicit double precision (a-h,o-z)

      parameter (nwpawc = 1000000)
      common /pawc/ hmemor(nwpawc)

      common /params/ sin2_tw,dmz
      common /flagcm/ istf,isign,pol
      common /xqgrid/ xxtab(100),qqtab(100),nxx,nqq
      common /abcoef/ ancl(-6:6,-1:1,3,3),ancr(-6:6,-1:1,3,3)
     
      external getfnc

      real t1,t2,t3

      dimension dn(-5:5),up(-5:5),dv(-5:5),uv(-5:5)

C--            bb  cb  sb  ub  db   g   d   u   s   c   b
      data dn / 1., 0., 1., 0., 1., 0., 1., 0., 1., 0., 1. /
      data up / 0., 1., 0., 1., 0., 0., 0., 1., 0., 1., 0. /
      data dv / 0., 0., 0., 0.,-1., 0., 1., 0., 0., 0., 0. /
      data uv / 0., 0., 0.,-1., 0., 0., 0., 1., 0., 0., 0. /

C--   Set parameters
      dmz       = 91.187
      sin2_tw   = 0.23124

C--   Choices
      istf     =  1       ! 1,2,3 for F2,FL,xF3
      isign    =  1       ! -1,+1 for electrons, positrons
      pol      =  0.      ! -1 .le. pol .le. +1 (polarization)

C--   Hbook initialisation
      call hlimit(nwpawc)

C--   x-Q2 table for plotting
      nxx        = 10
      xxtab(1)   = 1.D-3
      xxtab(nxx) = 0.9
      nqq        = 1
      qqtab(1)   = 10.D0
      call maketab

C--   Stop/print
      call ep_stop(1,0)

C--   Open stf file
      call timex(t1)
      call ep_open(20,'../fits/mbfit1m.stfnt2',ierr)

C--   Read dntype, uptype
C--   For F2, FL
      if(istf.eq.1.or.istf.eq.2) then
        call ep_read( 1, istf, 0, dn, ierr )
        call ep_read( 2, istf, 0, up, ierr )
      endif
C--   For xF3
      if(istf.eq.3) then
        call ep_read( 1, istf, 0, dv, ierr )
        call ep_read( 2, istf, 0, uv, ierr )
      endif

C--   Select error
      mode = 2

C--   Now for the result

      write(6,1000)

      call timex(t2)

      do ix = 1,nxx
        do iq = 1,nqq

          xx  = xxtab(ix)
          qq  = qqtab(iq)

          ff   = ep_ufun(getfnc,xx,qq,     ierr)
          ef   = ep_uerr(getfnc,xx,qq,mode,ierr)

          write(6,'(I5,3E11.3)') ix,xx,ff,ef

        enddo
      enddo
      call timex(t3)

      write(6,'(/'' Time spent init ='',F6.2)') t2-t1
      write(6,'( ''            loop ='',F6.2)') t3-t2
      write(6,'( ''           total ='',F6.2)') t3-t1

 1000 format(/'    #       x        fnc        err')

      end

C     ================================================
      double precision function getfnc(ipder,x,q,ierr)
C     ================================================

      implicit double precision (a-h,o-z)

C--   ep NC structure functions, see ZN 99-027

C--   ipder = fit parameter index
C--   istf  = 1,2,3 for F2, FL, xF3
C--   isign = electron sign
C--   pol   = electron polarisation
C--   x     = value of x
C--   q     = value of Q2
C--   ierr  = output error flag

      common /abcoef/ ancl(-6:6,-1:1,3,3),ancr(-6:6,-1:1,3,3)
      common /pqcoef/ pterm
      common /params/ sin2_tw,dmz
      common /flagcm/ istf,isign,pol

      logical first
      data first /.true./

C--   Initialization
      if(first) then
C--     Calculate anc(i,j,k,l) 
C--     i = [-6,6]   flavor index
C--     j = [-1,1]   electrons,positrons
C--     k = [ 1,3]   gamma, int, Z
C--     l = [ 1,3]   F2, FL, xF3
        call fncini
        first = .false.
      endif

      pq   = pterm*q/(q+dmz*dmz)
      facl = (1.+isign*pol)/2.
      facr = (1.-isign*pol)/2.

C--   Loop over dtype,utype
      ff    = 0.D0
      do iq = 1,2
        alq  = ancl(iq,isign,1,istf)    +
     +         ancl(iq,isign,2,istf)*pq +
     +         ancl(iq,isign,3,istf)*pq*pq
        arq  = ancr(iq,isign,1,istf)    +
     +         ancr(iq,isign,2,istf)*pq +
     +         ancr(iq,isign,3,istf)*pq*pq
        aq   = facl*alq + facr*arq
        ff   = ff + aq*ep_allf(iq,ipder,x,q,ierr)
      enddo

      getfnc = ff

      return
      end

C     =================
      subroutine fncini
C     =================

C--   Calculate anc(i,j,k,l), see ZN 99-027
C--   i = [-6,6]   flavor index
C--   j = [-1,1]   electrons,positrons
C--   k = [ 1,3]   gamma, int, Z
C--   l = [ 1,3]   F2, FL, xF3

      implicit double precision (a-h,o-z)

      common /abcoef/ ancl(-6:6,-1:1,3,3),ancr(-6:6,-1:1,3,3)
      common /pqcoef/ pterm
      common /params/ sin2_tw,dmz

      dimension e_q(-6:6)
      data e_q  /-2.,1.,-2.,1.,-2.,1.,0.,-1.,2.,-1.,2.,-1.,2./
      dimension t_q(-6:6)
      data t_q  /-1.,1.,-1.,1.,-1.,1.,0.,-1.,1.,-1.,1.,-1.,1./
      dimension e_e(-1:1)
      data e_e /-1.,0.,1./
      dimension t_e(-1:1)
      data t_e /-1.,0.,1./

      dimension v_q(-6:6),a_q(-6:6),v_e(-1:1),a_e(-1:1)

      do i = -6,6
        e_q(i) = e_q(i)/3.
        t_q(i) = t_q(i)/2.
        v_q(i) = t_q(i)-2.*e_q(i)*sin2_tw
        a_q(i) = t_q(i)
        do j = -1,1
          do k = 1,3
            do l = 1,3
              ancl(i,j,k,l) = 0.
              ancr(i,j,k,l) = 0.
            enddo
          enddo
        enddo
      enddo
      do i = -1,1
        t_e(i) = t_e(i)/2.
        v_e(i) = t_e(i)-2.*e_e(i)*sin2_tw
        a_e(i) = t_e(i)
      enddo

C--   loop over quark flavors
      do iq = -6,6
        eq = e_q(iq)
        vq = v_q(iq)
        aq = a_q(iq)
C--     loop over electrons,positrons
        do ie = -1,1,2
          ee = e_e(ie)
          ve = v_e(ie)
          ae = a_e(ie)
C--   A.  Left handed
C--     1.gamma term
          ancl(iq,ie,1,1) =     eq*eq
          ancl(iq,ie,1,2) =     eq*eq
          ancl(iq,ie,1,3) =     0.
C--     2.interference
          ancl(iq,ie,2,1) =  2.*ee*(ve+ae)*eq*vq
          ancl(iq,ie,2,2) =  2.*ee*(ve+ae)*eq*vq
          ancl(iq,ie,2,3) = +2.*ee*(ve+ae)*eq*aq
C--     3.Z echange
          ancl(iq,ie,3,1) =     (ve+ae)*(ve+ae)*(vq*vq+aq*aq)
          ancl(iq,ie,3,2) =     (ve+ae)*(ve+ae)*(vq*vq+aq*aq)
          ancl(iq,ie,3,3) = +2.*(ve+ae)*(ve+ae)*vq*aq
C--   B.  Right handed
C--     1.gamma term
          ancr(iq,ie,1,1) =     eq*eq
          ancr(iq,ie,1,2) =     eq*eq
          ancr(iq,ie,1,3) =     0.
C--     2.interference
          ancr(iq,ie,2,1) =  2.*ee*(ve-ae)*eq*vq
          ancr(iq,ie,2,2) =  2.*ee*(ve-ae)*eq*vq
          ancr(iq,ie,2,3) = -2.*ee*(ve-ae)*eq*aq
C--     3.Z echange
          ancr(iq,ie,3,1) =     (ve-ae)*(ve-ae)*(vq*vq+aq*aq)
          ancr(iq,ie,3,2) =     (ve-ae)*(ve-ae)*(vq*vq+aq*aq)
          ancr(iq,ie,3,3) = -2.*(ve-ae)*(ve-ae)*vq*aq
        enddo
      enddo

      pterm = 1./(4.*sin2_tw*(1.-sin2_tw))

      return
      end

C     ==================
      subroutine maketab
C     ==================

C--   Generate a logarithmic table of x and Q2 values

      implicit double precision (a-h,o-z)

      common /xqgrid/ xxtab(100),qqtab(100),nxx,nqq

      if(nxx.gt.1) then
        xmi  = xxtab(1)
        xma  = xxtab(nxx)
        xmil = log(xmi)
        xmal = log(xma)
        bw   = (xmal-xmil)/(nxx-1)
        do i = 1,nxx
          xl       = xmil + (i-1)*bw
          xxtab(i) = exp(xl)
        enddo
      endif

      if(nqq.gt.1) then
        qmi  = qqtab(1)
        qma  = qqtab(nqq)
        qmil = log(qmi)
        qmal = log(qma)
        bw   = (qmal-qmil)/(nqq-1)
        do i = 1,nqq
          ql       = qmil + (i-1)*bw
          qqtab(i) = exp(ql)
        enddo
      endif

      return
      end
